@@ -71,10 +71,21 @@ bsp=function(x, xk, qw, n=3L) {
 #'   Constraints are given as a 2-column matrix \code{(xe, ye)} where
 #'   for each xe, an ye value is imposed. If a list item is NULL, no constraint
 #'   is imposed on corresponding y column.
-#' @param monotone Numeric scalar, if monotone > 0, resulting B-spline
-#'   must be increasing;
-#'   if monotone < 0, B-spline must be decreasing; if monotone == 0 (default), no
-#'   constraint on monotonicity is imposed.
+#' @param monotone Numeric scalar or vector, if \code{monotone > 0}, resulting B-spline
+#'   weights must be increasing;
+#'   if \code{monotone < 0}, B-spline weights must be decreasing; if \code{monotone == 0} (default), no
+#'   constraint on monotonicity is imposed. If 'monotone' is a vector it
+#'   must be of length \code{ncol(y)}, in which case each component indicates
+#'   the constraint for corresponding column of y.
+#' @param positive Numeric scalar, if \code{positive > 0}, resulting B-spline weights
+#'   must be >= 0;
+#'   if \code{positive < 0}, B-spline weights must be decreasing; if \code{positive == 0} (default), no
+#'   constraint on positivity is imposed. If 'positive' is a vector it
+#'   must be of length \code{ncol(y)}, in which case each component indicates
+#'   the constraint for corresponding column of y.
+#' @param mat Numeric matrix of basis vectors, if NULL it is recalculated
+#'   by \code{bsc()}. If provided, it is the responsibility of the user
+#'   to ensure that this matrix be adequate to xki vector.
 #' @return Function, smoothing B-splines
 #'   respecting optional constraints (generated by \code{par2bsp()}).
 #' @details
@@ -110,7 +121,7 @@ bsp=function(x, xk, qw, n=3L) {
 #' @alias fitsmbsp
 #' @alias smbsp
 #' @export
-smbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0) {
+smbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0, positive=0, mat=NULL) {
     y=as.matrix(y)
     nx=length(x)
     nc=ncol(y)
@@ -134,7 +145,8 @@ smbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0) {
     
     nk=length(xk)
     nw=nk-n-1L
-    a=bsc(x, xk, n=n)
+    if (is.null(mat))
+        mat=bsc(x, xk, n=n)
     # equality constraints
     if (!is.null(lieq)) {
         stopifnot(length(lieq) == ncol(y))
@@ -146,25 +158,82 @@ smbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0) {
                 list(e=bsc(meq[,1L], n=n, xk), ce=meq[,2L])
             }
         })
+    } else {
+        liece=NULL
     }
-    if (monotone != 0) {
+    monotone[monotone > 0]=1.
+    monotone[monotone < 0]=-1.
+    positive[positive > 0]=1.
+    positive[positive < 0]=-1.
+    if (length(monotone) == 1L && nc > 1L)
+        monotone=rep(monotone, nc)
+    stopifnot(length(monotone) == ncol(y))
+    if (length(positive) == 1L && nc > 1L)
+        positive=rep(positive, nc)
+    stopifnot(length(positive) == ncol(y))
+    if (any(monotone != 0)) {
         # B-spline must be monotonously in- or de-creasing
         i=seq_len(nw-1L)
-        u=cbind(diag(x=-1., nrow=nw-1L), double(nw-1L))
-        u[cbind(i, i+1)]=1.
-        co=double(nw-1)
-        if (monotone < 0)
-            u=-u
+        um=cbind(diag(x=-1., nrow=nw-1L), double(nw-1L))
+        um[cbind(i, i+1)]=1.
+        com=double(nw-1)
     } else {
-        u=NULL
-        co=NULL
+        um=NULL
+        com=NULL
     }
-    qra=qr(a, LAPACK=TRUE)
-    if (!is.null(lieq) || monotone != 0) {
-        #browser()
-        qw=structure(vapply(seq_len(nc), function(ic) {nlsic::lsie_ln(qra, y[,ic], u=u, co=co, e=liece[[ic]]$e, ce=liece[[ic]]$ce)}, double(nw)), dim=c(nw, nc))
+    if (any(positive != 0)) {
+        # B-spline must be positive or negative
+        up=diag(x=1., nrow=nw)
+        cop=double(nw)
     } else {
-        qw=nlsic::ls_ln_svd(a, y)
+        up=NULL
+        cop=NULL
+    }
+    # u, co for all present combinations of monotone and positive
+    uco=list()
+    for (m in names(table(monotone))) {
+        if (is.null(uco[[m]]))
+            uco[[m]]=list()
+        if (m == "-1") {
+            u1=-um
+            co1=com
+        } else if (m == "1") {
+            u1=um
+            co1=com
+        } else {
+            u1=NULL
+            co1=NULL
+        }
+        for (p in names(table(positive))) {
+            if (is.null(uco[[m]][[p]]))
+                uco[[m]][[p]]=list()
+            if (p == "-1") {
+                u2=-up
+                co1=cop
+            } else if (p == "1") {
+                u2=up
+                co2=cop
+            } else {
+                u2=NULL
+                co2=NULL
+            }
+            uco[[m]][[p]]$u=rbind(u1, u2)
+            uco[[m]][[p]]$co=c(co1, co2)
+        }
+    }
+    monotone=as.character(monotone)
+    positive=as.character(positive)
+    qra=qr(mat, LAPACK=TRUE)
+    if (!is.null(lieq) || any(monotone != 0) || any(positive != 0)) {
+        #browser()
+        qw=structure(vapply(seq_len(nc), function(ic) {
+            m=monotone[ic]
+            p=positive[ic]
+            nlsic::lsie_ln(qra, y[,ic], u=uco[[m]][[p]]$u, co=uco[[m]][[p]]$co, e=liece[[ic]]$e, ce=liece[[ic]]$ce)
+        }, double(nw)), dim=c(nw, nc), dimnames=list(NULL, colnames(y)))
+        #browser()
+    } else {
+        qw=nlsic::ls_ln_svd(mat, y)
     }
     par2bsp(n, qw, xk)
 }
@@ -187,20 +256,18 @@ smbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0) {
 #'  # fit broken line with linear B-splines
 #'  x1=seq(0, 1, length.out=11)
 #'  x2=seq(1, 3, length.out=21)
-#'  x3=seq(3, 4, len=11)
+#'  x3=seq(3, 4, length.out=11)
 #'  y1=x1+rnorm(x1, sd=0.1)
 #'  y2=-2+3*x2+rnorm(x2, sd=0.1)
 #'  y3=4+x3+rnorm(x3, sd=0.1)
-#'  x=c(x1, x2[-1], x3[-1])
-#'  y=c(y1, y2[-1], y3[-1])
+#'  x=c(x1, x2, x3)
+#'  y=c(y1, y2, y3)
 #'  plot(x, y)
-#'  if (requireNamespace("numDeriv", quietly=TRUE)) {
-#'     f=fitsmbsp(x, y, n=1, nki=2)
-#'     lines(x, f(x))
-#'  }
+#'  f=fitsmbsp(x, y, n=1, nki=2)
+#'  lines(x, f(x))
 #' @importFrom nlsic nlsic lsi_ln
 #' @export
-fitsmbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0, control=list()) {
+fitsmbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0, positive=0, control=list()) {
     np=length(x)
     y=as.matrix(y)
     stopifnot(nrow(y) == np)
@@ -208,6 +275,8 @@ fitsmbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0, control=l
         xki=iknots(x, y, n=n, nki=nki)
     }
     nki=length(xki)
+    if (nki == 0L)
+        return(smbsp(x, y, n=n, xki=NULL, nki=0, lieq, monotone, positive))
     # inequalities
     # p[1] >= tail(x_1, 1)+epsx; p[i]+epsx <= p[i+1]; p[nki]+epsx <= x_n
     dx=diff(x)
@@ -220,13 +289,28 @@ fitsmbsp=function(x, y, n=3L, xki=NULL, nki=1L, lieq=NULL, monotone=0, control=l
     i=seq_len(nki-1L)
     u[cbind(i+1L, i)]=-1.
     u[cbind(i+1L, i+1L)]=1.; co[i+1L]=epsx
-    u[nki+1L, nki]=-1.; co[nki+1]=-x[np]-epsx
-    fit=nlsic::nlsic(xki, function(xki, cjac)
-        list(res=smbsp(x, y, n=n, xki=xki, nki=0, lieq, monotone)(x)-y),
+    u[nki+1L, nki]=-1.; co[nki+1]=-x[np]+epsx
+    x1=rep(x[1L], n+1L)
+    x2=rep(x[np], n+1L)
+    fresid=function(xki, cjac) {
+        if (cjac) {
+            li=bsc(x, c(x1, xki, x2), n=n, cjac=TRUE)
+            f=smbsp(x, y, n=n, xki=xki, nki=0, lieq=lieq, monotone=monotone, positive=positive, mat=li$mat)
+            res=f(x)-y
+            jac=jacw(li$jac, bsppar(f)$qw)[,,(n+1)+seq_len(nki), drop=FALSE]
+            dim(jac)=c(np*ncol(y), nki)
+            list(res=res, jacobian=jac)
+        } else {
+            list(res=smbsp(x, y, n=n, xki=xki, nki=0, lieq=lieq, monotone=monotone, positive=positive)(x)-y)
+        }
+    }
+    if (is.null(control$errx))
+        control$errx=epsx*0.1
+    fit=nlsic::nlsic(xki, fresid,
         u=u, co=co, control=control, flsi=nlsic::lsi_ln)
     if (fit$error != 0)
         stop(fit$mes)
-    smbsp(x, y, n=n, xki=fit$par, nki=0, lieq, monotone)
+    smbsp(x, y, n=n, xki=fit$par, nki=0, lieq=lieq, monotone=monotone, positive=positive)
 }
 
 #' Derivative of B-spline
@@ -255,18 +339,56 @@ dbsp=function(f, nderiv=1L) {
     n=e$n
     stopifnot(n >= nderiv)
     xkn=xk[c(-1L, -length(xk))]
-    qwn=n*arrApply::arrApply(qw, 1L, "diff")/diff(xkn, lag=n)
-    dimnames(qwn)=dimnames(qw)
+    qwn=arrApply::arrApply(qw, 1L, "diff")/(diff(xkn, lag=n)/n)
+    colnames(qwn)=colnames(qw)
     #browser()
     res=par2bsp(n-1L, qwn, xkn)
-
     if (nderiv == 1L) {
         res
     } else {
         dbsp(res, nderiv=nderiv-1L)
     }
 }
-
+#' Indefinite integral of B-spline
+#'
+#' @param f Function, B-spline such as returned by \code{smbsp()} or \code{par2bsp()}
+#' @param const Numeric scalar or vector of length \code{ncol(qw)} where
+#'   qw is weight matrix of f. Defines starting value of weights for indefinite
+#'   integral (0 by default).
+#' @param nint Integer scalar >= 0, defines how many times to take integral (1 by default)
+#' @return Function calculating requested integral
+#' @details
+#' If f is B-spline, then following identity is held: dbsp(ibsp(f)) is identical to f.
+#' Generally, it does not work in the other sens: ibsp(dbsp(f)) is not f
+#' but not very far. If we can get an appropriate constant C=f(min(x)) then
+#' we can assert that ibsp(dbsp(f), const=C) is the same as f.
+#' @export
+ibsp=function(f, const=0, nint=1L) {
+    stopifnot(nint >= 0L)
+    if (nint == 0L)
+        return(f)
+    e=environment(f)
+    qw=e$qw
+    stopifnot(length(const) == 1L || length(const) == ncol(qw))
+    xk=e$xk
+    n=e$n
+    n1=n+1L
+    xkn=xk[c(1L, seq_along(xk), length(xk))]
+    qwn=rbind(0, arrApply::arrApply(qw*diff(xk, lag=n1)/n1, 1L, "cumsum"))
+    if (length(const) == 1L) {
+        qwn=qwn+const
+    } else {
+        qwn=arrApply::arrApply(qwn, 2, "addv", v=const)
+    }
+    colnames(qwn)=colnames(qw)
+    #browser()
+    res=par2bsp(n+1L, qwn, xkn)
+    if (nint == 1L) {
+        res
+    } else {
+        ibsp(res, nint=nint-1L, const)
+    }
+}
 #' Retrieve parameters of B-splines
 #'
 #' @param f Function, B-splines such that returned by par3bsp(), smbsp(), ...
@@ -293,7 +415,7 @@ bsppar=function(f) {
 par2bsp=function(n, qw, xk)
     local({
         n=n
-        qw=qw
+        qw=as.matrix(qw)
         xk=xk
         function(x, select) {
             if (base::missing(select)) bsp(x, xk, qw, n=n) else
@@ -316,7 +438,7 @@ diffn=function(m, ndiff=1L) {
     stopifnot(nrow(m) > ndiff)
     nr=nrow(m)
     d=arrApply::arrApply(m, 1L, "diff")
-    res=cbind(m[-nr,1L]+0.5*d[,1L], d[,-1L]/d[,1L])
+    res=cbind(m[-nr,1L]+0.5*d[, 1L, drop=FALSE], d[, -1L, drop=FALSE]/d[,1L])
     if (ndiff > 1L) {
         res=diffn(res, ndiff-1L)
     }
@@ -341,16 +463,23 @@ iknots=function(x, y, nki=1L, n=3L) {
     if (nki == 0L)
         return(double(0L))
     y=as.matrix(y)
-    dxy=diffn(cbind(x, y), n+1L)
+    xy=cbind(x, y)
+    dubx=duplicated(x)
+    if (any(dubx)) {
+        ix=split(seq_along(x), x)
+        xy=t(vapply(ix, function(ii) if (length(ii) > 1L) colMeans(xy[ii,,drop=FALSE]) else xy[ii,], xy[1L,]))
+    }
+    dxyn=diffn(xy, n)
+    dxy=diffn(dxyn, 1L)
     dy=dxy[,-1L, drop=FALSE]
-    xtv=diffn(cbind(x, x), n)[,1L]
-    tv=rbind(0, arrApply::arrApply(abs(dy)*diff(x, lag=n+1), 1, "cumsum"))
+    xtv=dxyn[,1L]
+    tv=rbind(0, arrApply::arrApply(abs(dy)*diff(xtv), 1, "cumsum"))
     tv=arrApply::arrApply(tv, 2L, "multv", v=1./tv[nrow(tv),])
     tv[!is.finite(tv)]=0.
     tv=rowSums(tv)
     tv=tv/tv[length(tv)]
     ra=range(xtv)
-    ftv=smbsp(xtv, tv, xki=xtv[1]+diff(ra)*seq(0, 1, len=12)[2:11], n=1L, lieq=list(cbind(ra,0:1)), monotone=1)
+    ftv=smbsp(xtv, tv, xki=xtv[1]+diff(ra)*seq(0, 1, length.out=12)[2:11], n=1L, lieq=list(cbind(ra,0:1)), monotone=1)
     par=bsppar(ftv)
     if (FALSE) {
         print(c("knots qw=", par$qw))
